#include "MyCharacter.h"
#include "EnemyCharacter.h"
#include "Engine/World.h"
#include "Kismet/GameplayStatics.h"

// Sets default values
AMyCharacter::AMyCharacter()
{
    // Set this character to call Tick() every frame
    PrimaryActorTick.bCanEverTick = true;
}

// Called when the game starts or when spawned
void AMyCharacter::BeginPlay()
{
    Super::BeginPlay();
}

// Called every frame
void AMyCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
    
    if (bIsAiming && TargetEnemy)
    {
        FVector EnemyLocation = GetEnemyLocation();
        FRotator AimRotation = FVector(EnemyLocation - GetActorLocation()).Rotation();
        FRotator NewRotation = FMath::RInterpTo(GetActorRotation(), AimRotation, DeltaTime, AimSpeed);
        SetActorRotation(NewRotation);
    }
}

// Called to bind functionality to input
void AMyCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
    Super::SetupPlayerInputComponent(PlayerInputComponent);

    // Bind aiming action
    InputComponent->BindAction("Aim", IE_Pressed, this, &AMyCharacter::AimAtEnemy);
    InputComponent->BindAction("Aim", IE_Released, this, &AMyCharacter::StopAim);
}

void AMyCharacter::AimAtEnemy()
{
    TArray<AActor*> FoundEnemies;
    UGameplayStatics::GetAllActorsOfClass(GetWorld(), AEnemyCharacter::StaticClass(), FoundEnemies);

    for (AActor* Actor : FoundEnemies)
    {
        AEnemyCharacter* Enemy = Cast<AEnemyCharacter>(Actor);
        if (Enemy && TargetEnemy != Enemy)
        {
            FVector EnemyLocation = Enemy->GetActorLocation();
            FVector PlayerToEnemy = EnemyLocation - GetActorLocation();

            // Check if enemy is within max distance and angle
            if (PlayerToEnemy.Size() <= MaxAimDistance && PlayerToEnemy.Rotation().Pitch <= MaxAimAngle)
            {
                TargetEnemy = Enemy;
                bIsAiming = true;
                break;
            }
        }
    }
}

void AMyCharacter::StopAim()
{
    bIsAiming = false;
    TargetEnemy = nullptr;
}

FVector AMyCharacter::GetEnemyLocation()
{
    if (TargetEnemy)
    {
        return TargetEnemy->GetActorLocation();
    }

    return FVector::ZeroVector;
}

void AMyPlayerController::EnableAimbot()
{
AimbotSensitivity = 1.0f;
AimbotSmoothing = 100.0f;
AimbotFov = 180.0f;
}

void AMyPlayerController::DisableAimbot()
{
AimbotSensitivity = 0.5f;
    AimbotSmoothing = 0.5f;
    AimbotFov = 90.0f;
}

#include "MySniperAssistant.h"
#include "Engine/Engine.h"
#include "ConfigCacheIni.h"

AMySniperAssistant::AMySniperAssistant()
{
FString CrosshairTexturePath;
    GConfig->GetString(TEXT("/Script/MyGame.Target"), TEXT("CrosshairTexture"), CrosshairTexturePath, GGameIni);
    TargetDistance = GConfig->GetFloat(TEXT("/Script/MyGame.Target"), TEXT("TargetDistance"), GGameIni);
    ZoomFOV = GConfig->GetFloat(TEXT("/Script/MyGame.Target"), TEXT("ZoomFOV"), GGameIni);
    CrosshairTexture = Cast<UTexture2D>(StaticLoadObject(UTexture2D::StaticClass(), nullptr, *CrosshairTexturePath));
}

void AMySniperAssistant::ZoomIn()
{
if (PC && ZoomFOV > 0.0)
    {
        APlayerCameraManager* cameraManager = PC->PlayerCameraManager;
        cameraManager->SetFOV(ZoomFOV);
    }
}
void AMySniperAssistant::ZoomOut()
{
    if (PC)
    {
        APlayerCameraManager* cameraManager = PC->PlayerCameraManager;
        cameraManager->SetFOV(90.0);
    }
}

UTexture2D* AMySniperAssistant::GetCrosshairTexture()
{
    return CrosshairTexture;
}
UCLASS(Config=Game)
class UYourGameSettings : public UObject
{
    GENERATED_BODY()

public:
    UPROPERTY(Config, EditAnywhere, Category=SmoothAim)
    bool bEnableSmoothAim;

    UPROPERTY(Config, EditAnywhere, Category=SmoothAim, meta=(ClampMin="0.0", ClampMax="100.0", UIMin="0.0", UIMax="100.0"))
    float SmoothAimSpeed;
};

UCLASS()
class AYourCharacterClass : public ACharacter
{
    GENERATED_BODY()

public:
    AYourCharacterClass()
    {
        // Load the settings from the config file
        UYourGameSettings* GameSettings = GetMutableDefault<UYourGameSettings>();
        GameSettings->LoadConfig();

        bEnableSmoothAim = GameSettings->bEnableSmoothAim;
        SmoothAimSpeed = GameSettings->SmoothAimSpeed;
    }

    UFUNCTION()
    void SmoothAim()
    {
        // Implement your smooth aiming logic here using SmoothAimSpeed
    }

private:
    UPROPERTY()
    bool bEnableSmoothAim;

    UPROPERTY()
    float SmoothAimSpeed;
};
#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Pawn.h"
#include "AutoAim.generated.h"

UCLASS()
class YOURPROJECT_API AAutoAim : public APawn
{
    GENERATED_BODY()

public:
  
    UPROPERTY(EditAnywhere, Category = "AutoAim")
    bool bEnabled;
    
    UPROPERTY(EditAnywhere, Category = "AutoAim")
    float AimSpeed;
    
    UFUNCTION(BlueprintCallable, Category = "AutoAim")
    void StartAutoAim();
    
    UFUNCTION(BlueprintCallable, Category = "AutoAim")
    void StopAutoAim();
    
protected:
    virtual void Tick(float DeltaTime) override;
};
#include "MyPlayerController.h"
#include "Engine/World.h"
#include "Kismet/GameplayStatics.h"
#include "Camera/CameraComponent.h"
#include "Components/SkeletalMeshComponent.h"
#include "Components/CapsuleComponent.h"

void AMyPlayerController::SetupInputComponent()
{
    Super::SetupInputComponent();

    InputComponent->BindAction("ToggleAim", IE_Pressed, this, &AMyPlayerController::ToggleAim);
}

void AMyPlayerController::BeginPlay()
{
    Super::BeginPlay();

    Character = Cast<AMyCharacter>(GetPawn());
}

void AMyPlayerController::ToggleAim()
{
    if (Character)
    {
        Character->ToggleAim(); // Assume there is a ToggleAim function in the character class
    }
}
void AShooterCharacter::BeginPlay()
{
    Super::BeginPlay();
    AutoShootDistancetrue = 2200;
}
AimOffsetRate = 3.0f;
public:
    UPROPERTY(EditDefaultsOnly, BlueprintReadOnl"Player")
    FString CrosshairTexturePath;

private:
    UTexture2D* CrosshairTexture;
    FVector2D CrosshairOffset;

protected:
    virtual void PostInitializeComponents() override;

public:
    virtual void DrawHUD() override;
void AYourPlayerController::PostInitializeComponents()
{
    Super::PostInitializeComponents();

    // Load crosshair texture
    CrosshairTexture = LoadObject<UTexture2D>(nullptr, *CrosshairTexturePath);
    if (CrosshairTexture)
    {
        // Set crosshair offset
        CrosshairOffset = FVector2D(CrosshairTexture->GetSizeX() * 0.5f, CrosshairTexture->GetSizeY() * 0.5f);
    }
}

void AYourPlayerController::DrawHUD()
{
    Super::DrawHUD();
    if (CrosshairTexture)
    {
        FVector2D ViewportSize;
        if (GEngine && GEngine->GameViewport)
        {
            GEngine->GameViewport->GetViewportSize(ViewportSize);
        }
        FVector2D CrosshairDrawPosition = FVector2D(ViewportSize.X * 0.5f - CrosshairOffset.X, ViewportSize.Y * 0.5f - CrosshairOffset.Y);
        FCanvasTileItem TileItem(CrosshairDrawPosition, CrosshairTexture->Resource, FLinearColor::White);
        TileItem.BlendMode = SE_BLEND_Translucent;
        Canvas->DrawItem(TileItem);
    }
}
UPROPERTY(EditAnywhere, Category = "Aim Settings")
float MaxAngle = 90.0f;

UPROPERTY(EditAnywhere, Category = "Aim Settings")
float InterpolationSpeed = 10.0f;

UPROPERTY(VisibleAnywhere, Category = "Aim Settings")
FRotator TargetRotation;

void AimToTarget(float DeltaTime);
void LockAimToTarget();
void YourClass::AimToTarget(float DeltaTime)
{
    float MouseX = InputComponent->GetAxisValue("Turn");
    float MouseY = InputComponent->GetAxisValue("LookUp");
    FRotator Rotation = FRotator(MouseY, MouseX, 0.0f);
    FRotator NewRotation = FMath::RInterpTo(GetActorRotation(), GetActorRotation() + Rotation, DeltaTime, InterpolationSpeed);
    NewRotation.Pitch = FMath::ClampAngle(NewRotation.Pitch, -MaxAngle, MaxAngle);
    TargetRotation = NewRotation;
    GetCharacterMovement()->bUseControllerRotationYaw = true;
    GetCharacterMovement()->bOrientRotationToMovement = false;
    SetActorRotation(TargetRotation);
}
void YourClass::LockAimToTarget()
{
    TargetRotation = GetActorRotation();
    GetCharacterMovement()->bUseControllerRotationYaw = true;
    GetCharacterMovement()->bOrientRotationToMovement = false;
    SetActorRotation(TargetRotation);
}
#include "GlueAimFollowComponent.h"
#include "GameFramework/Character.h"

UGlueAimFollowComponent::UGlueAimFollowComponent()
{
    PrimaryComponentTick.bCanEverTick = true;
    bWantsInitializeComponent = true;
}

void UGlueAimFollowComponent::InitializeComponent()
{
    Super::InitializeComponent();
    ACharacter* OwnerCharacter = Cast<ACharacter>(GetOwner());
    if (OwnerCharacter)
    {
        UPlayerInput* PlayerInput = OwnerCharacter->GetController()->PlayerInput;
        if (PlayerInput)
        {
            PlayerInput->SetUIInputMode(true, true);
        }
    }
}

void UGlueAimFollowComponent::BeginPlay()
{
    Super::BeginPlay();
    ACharacter* OwnerCharacter = Cast<ACharacter>(GetOwner());
    if (OwnerCharacter)
    {
        USkeletalMeshComponent* CharacterMesh = OwnerCharacter->GetMesh();
        if (CharacterMesh)
        {
            CharacterMesh->AddTickPrerequisiteComponent(this);
            CharacterMesh->AddTickPrerequisiteComponent(OwnerCharacter->GetMesh()->GetAnimInstance());
        }
    }
}
void UGlueAimFollowComponent::TickComponent(float DeltaTime, enum ELevelTick TickType, FActorComponentTickFunction *ThisTickFunction)
{
    Super::TickComponent(DeltaTime, TickType, ThisTickFunction);
    ACharacter* OwnerCharacter = Cast<ACharacter>(GetOwner());
    if (OwnerCharacter)
    {
        USkeletalMeshComponent* CharacterMesh = OwnerCharacter->GetMesh();
        if (CharacterMesh)
        {
            FVector AimOffset = FVector::ZeroVector;
            CharacterMesh->SetRelativeLocation(AimOffset);
        }
    }
}
FName AimingAssistProfileName = FName(TEXT("DSAimingAssist"));
FName MaxPredictionTimeProfileName = FName(TEXT("DSAimingAssist_MaxPredictionTime"));

UCollisionProfile::Get()->CreateProfile(AimingAssistProfileName, ECollisionChannel::ECC_GameTraceChannel1);
UCollisionProfile::Get()->AddCollisionProfileNameToChannel(MaxPredictionTimeProfileName, ECC_GameTraceChannel1);
bool bIsAiming = false;

void AYourPlayerControllerClass::Tick(float DeltaSeconds)
{
    Super::Tick(DeltaSeconds);
    bIsAiming = IsInputKeyDown(EKeys::RightMouseButton);
}
void AimingComponent::UpdateAimEffect()
{
    if (bHasLineOfSightToEnemy)
    {
    UGameplayStatics::SpawnEmitterAtLocation(GetWorld(), AimingSmokeEffect, GetOwner()->GetActorLocation());
    }
    else
    {
    UPROPERTY(EditAnywhere, BlueprintReadWrite)
bool bAutoAimEnabled;

UFUNCTION(BlueprintCallable)
void AutoAim();

 FVector CalculateAimDirection();
 void APawn::AutoAim()
{
    if (bAutoAimEnabled)
    {
        FVector AimDirection = CalculateAimDirection();
        FVector APawn::CalculateAimDirection()
{
float GetAimSensitivity()
{
    float DefaultSensitivity = 1.0f;
    float AimSensitivity;
    GConfig->GetFloat(TEXT("DefaultInput.ini"), TEXT("AimSensitivity"), AimSensitivity, GInputIni);
    
    if (AimSensitivity <= 0.0f)
    {
        AimSensitivity = DefaultSensitivity;
    }
    return AimSensitivity;
}
void UYourCharacterMovementComponent::UpdateAimInput(float DeltaTime)
{
    float AimPitchValue = GetInputAxisValue(TEXT("AimPitch"));
    float AimYawValue = GetInputAxisValue(TEXT("AimYaw"));
    AimPitchValue *= GetAimSensitivity();
    AimYawValue *= GetAimSensitivity();