#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "AimAssistComponent.generated.h"

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class YOURPROJECT_API UAimAssistComponent : public UActorComponent
{
	GENERATED_BODY()

public:	
	// Sets default values for this component's properties
	UAimAssistComponent();

protected:
	// Called when the game starts
	virtual void BeginPlay() override;

public:	
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

private:
	// The maximum distance of the aim assist effect
	UPROPERTY(EditAnywhere, Category = "Aim Assist", meta = (UIMin = "0.0", UIMax = "1000.0"))
	float MaxAssistDistance;

	// The maximum strength of the aim assist effect
	UPROPERTY(EditAnywhere, Category = "Aim Assist", meta = (UIMin = "0.0", UIMax = "1.0"))
	float MaxAssistStrength;

	// The strength of the aim assist effect at close distances
	UPROPERTY(EditAnywhere, Category = "Aim Assist", meta = (UIMin = "0.0", UIMax = "1.0"))
	float CloseAssistStrength;

	// The strength of the aim assist effect at far distances
	UPROPERTY(EditAnywhere, Category = "Aim Assist", meta = (UIMin = "0.0", UIMax = "1.0"))
	float FarAssistStrength;

	// The target aim assist location
	FVector TargetLocation;

	// The target aim assist strength
	float TargetStrength;

public:
	// Called to set the target aim assist location
	UFUNCTION(BlueprintCallable, Category = "Aim Assist")
	void SetTargetLocation(const FVector& Location);

private:
	// Calculate the aim assist strength based on the target location and distance
	void CalculateAssistStrength(const FVector& CurrentAimLocation);
};