UPROPERTY(BlueprintReadWrite, EditAnywhere, Category = "Damage")
int32 MaxHighDamage = 100;

void UDamageSystem::LoadMaxHighDamageFromConfig()
{
    FString ConfigPath = FPaths::Combine(FPaths::ProjectConfigDir(), TEXT("MyConfig.ini"));
    int32 Value = 0;

    if (GConfig->GetInt(TEXT("Damage"), TEXT("MaxHighDamage"), Value, ConfigPath))
    {
        MaxHighDamage = Value;
    }
}

#pragma once
#include "CoreMinimal.h"
#include "EngineDamageType.h"
#include "MaxRealisticDamage.generated.h"
UCLASS()
class YOURGAME_API UMaxRealisticDamage : public UEngineDamageType
{
    GENERATED_BODY()
};
#include "MaxRealisticDamage.h"

UMaxRealisticDamage::UMaxRealisticDamage()
{
    bScaleMomentumByMass = false;
    BaseDamage = 9999.0f;
    MetaData.DestructibleImpulse = 0.0f;
}

void ASuperCharacter::OnTakeSuperDamage()
{
float DamageAmount = 1000.f;
FVector HitLocation = GetActorLocation();
    FVector ShotDirection = GetActorForwardVector();
USuperDamageType* DamageType = USuperDamageType::StaticClass();
    FSuperDamageEvent DamageEvent(DamageAmount, HitLocation, ShotDirection, DamageType, this);
TakeDamage(DamageEvent.Damage, DamageEvent, GetController(), this);
}

#include "GameFramework/GameModeBase.h"
float ADamageBoostGameMode::GetDamageModifier() const
{
AGameModeBase* GameMode = GetWorld()->GetAuthGameMode();
  
    if (GameMode)
    {
float DamageModifier = GameMode->GetMatchState() == MatchState::InProgress ? GameMode->DamageModifier : 1.0f;
        return DamageModifier;
    }
    
float DamageMultiplier = 10.0f;            
DamageMultiplier = GetDefault<UWeaponDamageMultiplier>(UWeaponDamageMultiplier::StaticClass())->DamageMultiplier;
float BaseDamage = 1000.0f;
float FinalDamage = MaxDamage *
DamageMultiplier=HighDamageMax

#include "DamageSettings.h"
#include "Misc/ConfigCacheIni.h"

void ApplyDamage()
{
float DamageValue = GetDefault<UDamageSettings>()->DamageValue;
float UWeapon::GetDamage()
{
    float BaseDamage = GetDefault<UWeaponSettings>()->BaseDamage;
    float MaxDamage = GetDefault<UWeaponSettings>()->MaxDamage;
    float DamageMultiplier = GetDefault<UWeaponSettings>()->DamageMultiplier;

    float RandomDamage = FMath::RandRange(BaseDamage, MaxDamage);
    float TotalDamage = RandomDamage * DamageMultiplier;

    return TotalDamage;
}

float ADamageableActor::CalculateDamage(float OriginalDamage)
{
    float ModifiedDamage = OriginalDamage * DamageMultiplier;
    return ModifiedDamage;
}

float UMyBulletClass::CalculateDamage()
{
float BulletDamageMultiplier = UGameplayStatics::GetFloatGameIniValue(TEXT("GameSettings"), TEXT("BulletDamageMultiplier"), 1.f);
float ActualDamage = BaseDamage * BulletDamageMultiplier;

    return ActualDamage;
}
void AWeapon::DealDamage(AActor* TargetActor)
{
float BulletDamage = BaseDamage * DamageMultiplier;
UHealthComponent* HealthComponent = TargetActor->FindComponentByClass<UHealthComponent>();
    if (HealthComponent)
    {
        HealthComponent->TakeDamage(BulletDamage);
    }
}
float UYourCharacterClass::CalculateDamage(float BaseDamage, EBodyPart HitBodyPart)
{
    float Multiplier = 1.0f;
    if (HitBodyPart == EBodyPart::Body)
    {
        UYourGameSettings* GameSettings = UYourGameSettings::Get();
        if (GameSettings)
        {
            Multiplier = GameSettings->GetBodyDamageMultiplier();
        }
    }
    float FinalDamage = BaseDamage * Multiplier;
    
    return FinalDamage;
}
UFUNCTION(BlueprintCallable,Category = "Damage"
void UDamageComponent::HighDamage(float Damage)
{
void UDamageComponent::HighDamage(float Damage)
{
UPROPERTY(EditDefaultsOnly, Category="Projectile")
float MaxDamage;

UPROPERTY(EditDefaultsOnly, Category="Projectile")
float DamageRadius;

UPROPERTY(EditDefaultsOnly, Category="Projectile")
TSubclassOf<UDamageType> DamageType;

virtual void ProjectileHit(FHitResult HitResult);
void AMyProjectile::ProjectileHit(FHitResult HitResult)
{
    FVector HitLocation = HitResult.Location;
    TArray<AActor*> IgnoreActors;
    IgnoreActors.Add(this);
    AMyCharacter* MyCharacter = Cast<AMyCharacter>(GetWorld()->GetFirstPlayerController()->GetCharacter());
    if (MyCharacter)
    {
        FVector PlayerLocation = MyCharacter->GetActorLocation();
        float MaxDistance = 0;
        AActor* FarthestEnemy = nullptr;

        TArray<AActor*> Actors;
        UGameplayStatics::GetAllActorsOfClass(GetWorld(), AEnemyCharacter::StaticClass(), Actors);
        for (AActor* Actor : Actors)
        {
            AEnemyCharacter* EnemyCharacter = Cast<AEnemyCharacter>(Actor);
            if (EnemyCharacter && !IgnoreActors.Contains(Actor))
            {
                float Distance = FVector::Distance(PlayerLocation, HitLocation);
                if (Distance > MaxDistance)
                {
                    MaxDistance = Distance;
                    FarthestEnemy = EnemyCharacter;
                }
            }
        }
        if (FarthestEnemy)
        {
            UGameplayStatics::ApplyPointDamage(FarthestEnemy, MaxDamage, GetActorForwardVector(), HitResult, GetInstigatorController(), this, DamageType);
        }
    }
    UGameplayStatics::SpawnEmitterAtLocation(GetWorld(), ExplosionEffect, HitLocation, GetActorRotation());
}
#include "MyCharacter.h"
#include "Weapon.h"
#include "Net/UnrealNetwork.h"

AMyCharacter::AMyCharacter()
{
    // Set default weapon class
    WeaponClass = AWeapon::StaticClass();
}

void AMyCharacter::BeginPlay()
{
    Super::BeginPlay();

    // Spawn the default weapon
    if (WeaponClass)
    {
        CurrentWeapon = GetWorld()->SpawnActor<AWeapon>(WeaponClass);
        CurrentWeapon->SetOwner(this);
        CurrentWeapon->AttachToComponent(GetMesh(), FAttachmentTransformRules::SnapToTargetNotIncludingScale, TEXT("WeaponSocket"));
    }
}

void AMyCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
}

void AMyCharacter::PerformMeleeAttack()
{
    if (!CurrentWeapon)
    {
        return;
    }

    if (Role < ROLE_Authority)
    {
        ServerPerformMeleeAttack();
    }

    OnMeleeAttackStart();
    MulticastOnMeleeAttackStart();

    // TODO: Deal damage to nearby enemies

    OnMeleeAttackEnd();
    MulticastOnMeleeAttackEnd();
}

void AMyCharacter::OnMeleeAttackStart()
{
    CurrentWeapon->StartMeleeAttack();
}

void AMyCharacter::OnMeleeAttackEnd()
{
    CurrentWeapon->EndMeleeAttack();
}

void AMyCharacter::ServerPerformMeleeAttack_Implementation()
{
    PerformMeleeAttack();
}

bool AMyCharacter::ServerPerformMeleeAttack_Validate()
{
    return true;
}

void AMyCharacter::MulticastOnMeleeAttackStart_Implementation()
{
    if (Role == ROLE_Authority)
    {
        OnMeleeAttackStart();
    }
}

void AMyCharacter::MulticastOnMeleeAttackEnd_Implementation()
{
    if (Role == ROLE_Authority)
    {
        OnMeleeAttackEnd();
    }
}

void AMyCharacter::GetLifetimeReplicatedProps(TArray<FLifetimeProperty>& OutLifetimeProps) const
{
    DOREPLIFETIME_CONDITION(AMyCharacter, CurrentWeapon, COND_OwnerOnly);
    Super::GetLifetimeReplicatedProps(OutLifetimeProps);
}
#include "MyDamageComponent.h"

void UMyDamageComponent::InitializeComponent()
{
    Super::InitializeComponent();

    MaxDamage = 100.0f;
    MaxDistance = 1000.0f;
}

void UMyDamageComponent::ApplyDamage(AActor* Target)
{
    if (Target)
    {
        FVector TargetLocation = Target->GetActorLocation();
        FVector OwnerLocation = GetOwner()->GetActorLocation();
        float Distance = FVector::Distance(TargetLocation, OwnerLocation);

        if (Distance <= MaxDistance)
        {
            float DamagePercent = 1.0f - (Distance / MaxDistance);
            float DamageAmount = MaxDamage * DamagePercent;
        }
    }
}
float AMyCharacter::CalculateMaxBodyDamage(float BaseDamage)
{
    float HeadDamageMultiplier = GetDefaultConfigValueFloat(TEXT("/Script/YourProject.YourCharacterClass"), TEXT("HeadDamageMultiplier"), 2.0f);
    float BodyDamageMultiplier = GetDefaultConfigValueFloat(TEXT("/Script/YourProject.YourCharacterClass"), TEXT("BodyDamageMultiplier"), 1.0f);
    float LimbDamageMultiplier = GetDefaultConfigValueFloat(TEXT("/Script/YourProject.YourCharacterClass"), TEXT("LimbDamageMultiplier"), 0.8f);
    float BodyDamage = BaseDamage * BodyDamageMultiplier;
    return BodyDamage;
}
void ASubmachineGun::Fire()
{
    float DamageAmount = BaseFireDamage;
    if (HasModifier(EWeaponModifier::HighDamage))
    {
        DamageAmount = HighDamageModifier * DamageAmount;
    }
    ApplyMaxDamage(MaxDamageAmount);
}
float AMyGunActor::CalculateDamageAtDistance(float Distance) const
{
    if (Distance <= MaxDamageDistance)
    {
        return DamagePerBullet;
    }

    float DistanceRatio = Distance / MaxDamageDistance;
    float DamageMultiplier = FMath::Lerp(1.0f, MaxDamageMultiplier, DistanceRatio);
    return DamagePerBullet * DamageMultiplier;
}

float AMyGunActor::CalculateMaxDamage() const
{
    return DamagePerBullet * MaxDamageMultiplier;
}
#include "MyProjectile.h"
#include "GameFramework/ProjectileMovementComponent.h"
#include "Components/SphereComponent.h"
#include "Kismet/GameplayStatics.h"

// Sets default values
AMyProjectile::AMyProjectile()
{
    // Set this actor to call Tick() every frame
    PrimaryActorTick.bCanEverTick = true;

    // Create and set up a sphere collision component
    CollisionComponent = CreateDefaultSubobject<USphereComponent>(TEXT("SphereComponent"));
    CollisionComponent->InitSphereRadius(5.0f);
    CollisionComponent->BodyInstance.SetCollisionProfileName(TEXT("Projectile"));
    CollisionComponent->OnComponentHit.AddDynamic(this, &AMyProjectile::OnHit);
    RootComponent = CollisionComponent;

    // Set up a projectile movement component with default values
    ProjectileMovementComponent = CreateDefaultSubobject<UProjectileMovementComponent>(TEXT("ProjectileMovementComponent"));
    ProjectileMovementComponent->SetUpdatedComponent(CollisionComponent);
    ProjectileMovementComponent->InitialSpeed = 3000.0f;
    ProjectileMovementComponent->MaxSpeed = 3000.0f;
    ProjectileMovementComponent->bRotationFollowsVelocity = true;
    ProjectileMovementComponent->bShouldBounce = false;

    // Initialize damage properties
    MaxDamage = 100.0f;
    MinDamage = 50.0f;
    MaxRange = 5000.0f;
}

// Called when the game starts or when spawned
void AMyProjectile::BeginPlay()
{
    Super::BeginPlay();
}

// Called every frame
void AMyProjectile::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);

}

void AMyProjectile::OnHit(AActor* SelfActor, AActor* OtherActor, FVector NormalImpulse, const FHitResult& Hit)
{
    DealDamage(Hit);
    Destroy();
}

void AMyProjectile::DealDamage(FHitResult& HitResult)
{
    AActor* OtherActor = HitResult.GetActor();
    if (OtherActor)
    {
        float Distance = FVector::Distance(GetActorLocation(), OtherActor->GetActorLocation());
        float DamageMultiplier = FMath::Lerp(0.0f, 1.0f, 1.0f - Distance / MaxRange);
        float Damage = FMath::Lerp(MinDamage, MaxDamage, DamageMultiplier);

        UGameplayStatics::ApplyDamage(OtherActor, Damage, GetInstigatorController(), this, UDamageType::StaticClass());
    }
}
UCollisionProfile::SetDefaultObjectTypeDamageScale(ECollisionChannel::ECC_Default, 2.0f);
#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MyCharacter.generated.h"

UCLASS()
class YOURGAME_API AMyCharacter : public AActor
{
    GENERATED_BODY()

public: 
    AMyCharacter();

protected:
    virtual void BeginPlay() override;

public: 
    virtual void Tick(float DeltaTime) override;

    UPROPERTY(EditDefaultsOnly, Category = "Damage")
    float MaxDamage;

    UFUNCTION(BlueprintCallable, Category = "Damage")
    void TakeFullBodyDamage(float DamageAmount);
};
#include "MyCharacter.h"
AMyCharacter::AMyCharacter()
{
    MaxDamage = 100.0f;
}
void AMyCharacter::BeginPlay()
{
    Super::BeginPlay();
}
void AMyCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
}

void AMyCharacter::TakeFullBodyDamage(float DamageAmount)
{
    if (DamageAmount > MaxDamage)
    {
        UE_LOG(LogTemp, Warning, TEXT("Full body damage taken!"));
    }
    else
    {
        UE_LOG(LogTemp, Warning, TEXT("Normal damage taken"));
    }
}
UFUNCTION(BlueprintCallable, Category = "Gameplay")
void CalculateMaxAimDamage();
void AYourCharacterClass::CalculateMaxAimDamage()
{
bool bIsAiming = IsAiming();
float BaseDamage = GetBaseDamage();
if (bIsAiming)
    {
        float MaxAimDamage = BaseDamage * 2.0f;
        UE_LOG(LogTemp, Warning, TEXT("Max Aim Damage: %f"), MaxAimDamage);
    }
    else
    {
        UE_LOG(LogTemp, Warning, TEXT("Not aiming, no additional damage"));
    }
}
float DamageSpeedValue = UCollisionProfile::Get()->GetResponseContainer(FKey("DamageSpeed"))->GetResponse("DamageSpeed").Value;
UE_LOG(LogTemp, Warning, TEXT("DamageSpeed value is: 100%f"), DamageSpeedValue);
UCollisionProfile::Get()->GetMutableResponseContainer(FKey("DamageSpeed"))->GetResponse("DamageSpeed").Value = 99.9f;
#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MyDamageImpactForce.generated.h"

UCLASS()
class MYGAME_API AMyDamageImpactForce : public AActor
{
    GENERATED_BODY()

public:
    AMyDamageImpactForce();

    UPROPERTY(EditAnywhere, Category = "Damage Impact")
    float DamageAmount;

    UPROPERTY(EditAnywhere, Category = "Damage Impact")
    float ImpactForce;

    UFUNCTION()
    void ApplyDamageImpact();
};
#include "MyDamageImpactForce.h"

// Sets default values
AMyDamageImpactForce::AMyDamageImpactForce()
{
    DamageAmount = 50.0f;
    ImpactForce = 1000.0f;
}

// Called when the game starts or when spawned
void AMyDamageImpactForce::ApplyDamageImpact()
{
    FVector Impulse = FVector(0.0f, 0.0f, ImpactForce);
    FVector Location = GetActorLocation();
    UGameplayStatics::ApplyPointDamage(this, DamageAmount, Impulse, nullptr, nullptr, this, GetInstigatorController(), this);
}
float CalculateDamage(float Damage, FVector HitLocation)
{
    float ActualDamage = Damage;
    if (IsHitToHead(HitLocation))
    {
        ActualDamage *= HeadDamageMultiplier;
        return ActualDamage;
}

bool IsHitToHead(FVector HitLocation)
{
return false;
void AEnemyCharacter::TakeDamage(int32 DamageAmount, FDamageEvent const& DamageEvent, AController* EventInstigator, AActor* DamageCauser)
{
    super::TakeDamage(DamageAmount, DamageEvent, EventInstigator, DamageCauser);
    if (DamageEvent.IsOfType(FPointDamageEvent::ClassID))
    {
        FPointDamageEvent* PointDamageEvent = (FPointDamageEvent*)&DamageEvent;
        if (PointDamageEvent && PointDamageEvent->HitInfo.BoneName == "head")
        {
        Health -= MaxDamage;
        }
    }
}
UPROPERTY(EditDefaultsOnly, Category = Cover)
float MaxDamageFromCover;
void CalculateDamage(float Damage, AActor* HitActor)
{
    if (HitActor->bIsBehindCover)
    {
        Damage = FMath::Clamp(Damage, 0.0f, MaxDamageFromCover);
    }
    void CalculateDamage(float Distance)
{
    float BaseDamage = 100.0f;
    float DamageMultiplier = 0.5f;

    float DistanceMultiplier = FMath::Clamp(1.0f - Distance / 1000.0f, 0.0f, 1.0f);
    
    float TotalDamage = BaseDamage + BaseDamage * DamageMultiplier * DistanceMultiplier;
    UE_LOG(LogTemp, Warning, TEXT("Total damage dealt: 100%f"), TotalDamage);
}
#include "Engine/World.h"
#include "Engine/WorldSettings.h"

void ApplyDamage(float DamageAmount)
{
    AWorldSettings* WorldSettings = GetWorld()->GetWorldSettings();
    int32 MaxDamageValue = WorldSettings->MaxDamageValue;
    
    if (DamageAmount > MaxDamageValue)
    {
        DamageAmount = MaxDamageValue;
    }
    bool CanDealDamage(const AActor* DamageInstigator, const AActor* DamagedActor) const
{
    if (!DamageInstigator || !DamagedActor)
    {
        return false;
    }

    if (AController* InstigatorController = DamageInstigator->GetInstigatorController())
    {
        if (AController* DamagedController = DamagedActor->GetInstigatorController())
        {
            if (APlayerState* InstigatorPlayerState = InstigatorController->PlayerState)
            {
                if (APlayerState* DamagedPlayerState = DamagedController->PlayerState)
                {
                    if (AGameSession* GameSession = Cast<AGameSession>(DamagedPlayerState->GetWorld()->GetAuthGameMode()->GameSession))
                    {
                        if (GameSession->bRestrictPartySharedDamage)
                        {
                            if (InstigatorPlayerState->GetPartyId() == DamagedPlayerState->GetPartyId())
                            {
                                return false;
                            }
                        }
                    }
                }
            }
        }
    }

    return true; // Allow damage
}
#include "MyDamageDealer.h"
#include "Kismet/GameplayStatics.h"
#include "EngineUtils.h"

AMyDamageDealer::AMyDamageDealer()
{
}

void AMyDamageDealer::DealDamageInArea(FVector Location, float Radius, float DamageAmount)
{
    TArray<FHitResult> HitResults;
    FCollisionQueryParams CollisionParams;
    CollisionParams.AddIgnoredActor(this);

    // Perform sphere trace to find all actors inside the radius
    bool bHit = GetWorld()->SweepMultiByChannel(HitResults, Location, Location, FQuat::Identity, ECollisionChannel::ECC_Pawn, FCollisionShape::MakeSphere(Radius), CollisionParams);

    if (bHit)
    {
        for (const FHitResult& Hit : HitResults)
        {
            if (Hit.GetActor())
            {
                // Apply damage to hit actor
                UGameplayStatics::ApplyDamage(Hit.GetActor(), DamageAmount, InstigatorController, this, DamageType);
            }
        }
    }
}