UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Gameplay")
class UBoxComponent* HeadCollisionComponent;

UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Gameplay")
class UParticleSystem* ExplosionEffect;
AHeadActor::AHeadActor()
{
    HeadCollisionComponent = CreateDefaultSubobject<UBoxComponent>(TEXT("HeadCollisionComponent"));
    HeadCollisionComponent->SetCollisionEnabled(ECollisionEnabled::QueryAndPhysics);
    HeadCollisionComponent->SetCollisionObjectType(ECC_Pawn);
    HeadCollisionComponent->SetCollisionResponseToAllChannels(ECR_Ignore);
    HeadCollisionComponent->SetCollisionResponseToChannel(ECC_WorldStatic, ECR_Block);
    HeadCollisionComponent->SetCollisionResponseToChannel(ECC_WorldDynamic, ECR_Block);
    HeadCollisionComponent->SetCollisionResponseToChannel(ECC_Pawn, ECR_Block);
    HeadCollisionComponent->OnComponentHit.AddDynamic(this, &AHeadActor::OnHeadCollision);

    RootComponent = HeadCollisionComponent;
}
void AHeadActor::OnHeadCollision(UPrimitiveComponent* HitComponent, AActor* OtherActor, UPrimitiveComponent* OtherComp, FVector NormalImpulse, const FHitResult& Hit)
{    
if (OtherActor->IsA(AEnemyCharacter::StaticClass()))
    {
UGameplayStatics::SpawnEmitterAtLocation(GetWorld(), ExplosionEffect, GetActorLocation(), FRotator::ZeroRotator);
Destroy();
    }
}

UENUM(BlueprintType)
enum class EBodyPart : uint8
{
    Body,
    Head
};

UCLASS()
class AGun : public AActor
{
    GENERATED_BODY()

public:
    AGun()
    {
UConfigLoader::LoadConfigFile("GunSettings", *this);

        CurrentBullets = MaxBullets;
        CurrentHeadshots = 0;
    }

    UPROPERTY(EditAnywhere, Config, Category = "GunSettings")
    int32 MaxBullets;

    UPROPERTY(EditAnywhere, Config, Category = "GunSettings")
    int32 HeadshotMultiplier;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Gun")
    int32 CurrentBullets;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Gun")
    int32 CurrentHeadshots;

    UFUNCTION(BlueprintCallable, Category = "Gun")
    void Shoot(EBodyPart HitBodyPart)
    {
        if (CurrentBullets > 0)
        {
            CurrentBullets--;

            if (HitBodyPart == EBodyPart::Head)
            {
CurrentHeadshots++;
                ApplyDamage(HeadshotMultiplier);
            }
            else
            {
ApplyDamage(1);
            }
        }
    }

private:
    void ApplyDamage(int32 Damage)
    {                            

#pragma once

#include "CoreMinimal.h"
#include "Gameplay(2D)
#include "GameFramework/GameModeBase.h"
#include "OneShotOneKillGameMode.generated.h"

UCLASS()
class ONESHOTONEKILL_API AOneShotOneKillGameMode : public AGameModeBase
{
	GENERATED_BODY()

public:
	// Sets default values for this character's properties
	AOneShotOneKillGameMode();

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

public:
	// Called every frame
	virtual void Tick(float DeltaTime) override;

};
#include "OneShotOneKillGameMode.h"

void AOneShotOneKillGameMode::BeginPlay()
{
	Super::BeginPlay();
	
	// TODO: Add custom game logic here
}

void AOneShotOneKillGameMode::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
	
	// TODO: Add custom game logic here
}

void AMyCharacter::MaximizeHeadshot(FVector TargetLocation)
{
FVector HeadLocation = GetMesh()->GetBoneLocation(HeadBoneName);
FVector Direction = FVector(TargetLocation - HeadLocation).GetSafeNormal();
SetActorRotation(Direction.Rotation());
FRotator TargetRotator = Direction.Rotation() - GetActorRotation();
    GetMesh()->SetRelativeRotation(TargetRotator);
}

UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Components")
UBoxComponent* HeadCollisionComponent;
APlayerCharacter::APlayerCharacter()
{
HeadCollisionComponent = CreateDefaultSubobject<UBoxComponent>(TEXT("HeadCollisionComponent "));
HeadCollisionComponent->SetCollisionProfileName(TEXT("MaxHeadshot"));
HeadCollisionComponent->SetupAttachment(GetMesh(), FName("HeadSocket"));
HeadCollisionComponent->OnComponentHit.AddDynamic(this, &APlayerCharacter::OnHeadHit);
}
void APlayerCharacter::OnHeadHit(UPrimitiveComponent* HitComponent, AActor* OtherActor, UPrimitiveComponent* OtherComp, FVector NormalImpulse, const FHitResult& Hit)
{
if (OtherActor->IsA(AEnemyCharacter::StaticClass()) && HeadCollisionComponent->GetName() == "HeadCollisionComponent")
    {
UE_LOG(LogTemp, Warning, TEXT("MaxHeadshot!"));
    }
}    