UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Recoil")
bool bEnableNoRecoil;

UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Recoil", meta=(UIMin="0.0", UIMax="1.0"))
float NoRecoilStrength;
AMyGameMode::AMyGameMode()
{
    bEnableNoRecoil = false;
    NoRecoilStrength = 0.0f;
}
void AMyCharacter::Fire()
{
if (bEnableNoRecoil)
    {
ApplyNoRecoil();
    }
void AMyCharacter::ApplyNoRecoil()
{
FVector NoRecoilVector = FVector::ZeroVector;
    NoRecoilVector.X = FMath::RandRange(-NoRecoilStrength, NoRecoilStrength);
    NoRecoilVector.Y = FMath::RandRange(-NoRecoilStrength, NoRecoilStrength);
    NoRecoilVector.Z = FMath::RandRange(-NoRecoilStrength, NoRecoilStrength);
AddControllerPitchInput(NoRecoilVector.Y);
    AddControllerYawInput(NoRecoilVector.X);
AddCameraWorldOffset(NoRecoilVector);
}
if (GEngine)
{
GEngine->bReduceSystemLatency = true;
    GEngine->bUseFixedTimeStep = true;
    GEngine->FixedFrameRate = 30;
}                                
UGameUserSettings* GameUserSettings = GetGameUserSettings();
if (GameUserSettings != nullptr)
{
    GameUserSettings->bAlwaysReorientCamera = true;
    GameUserSettings->ApplySettings(false);
float MaxRecoilYaw;
float MaxRecoilPitch;
float RecoilRecoveryRate;
float CurrentRecoilYaw;
float CurrentRecoilPitch;    
/** The amount to reduce the horizontal recoil force by. */
UPROPERTY(EditAnywhere, Category = "Recoil")
float RecoilReductionAmount = 1.0f;
void AYourCharacterClass::ReduceHorizontalRecoil()
{
UMyRecoilComponent* RecoilComponent = FindComponentByClass<UMyRecoilComponent>();
if (RecoilComponent)
    {
FVector2D CurrentHorizontalRecoilForce = RecoilComponent->GetCurrentHorizontalRecoilForce();
CurrentHorizontalRecoilForce.X *= 1.0f - RecoilReductionAmount;
RecoilComponent->SetCurrentHorizontalRecoilForce(CurrentHorizontalRecoilForce);
    }
}    

#pragma once
#include "CoreMinimal.h"
#include "YourWeaponClass.h"
#include "MyWeaponClass.generated.h"
UCLASS()
class YOURPROJECT_API AMyWeaponClass : public AYourWeaponClass
{
	GENERATED_BODY()
public:
	AMyWeaponClass();
	UFUNCTION(BlueprintCallable, Category = "Weapon")
	void ReduceVerticalRecoil();
};
void APawn::OnPawnFire()
{
FVector forwardVector = GetActorForwardVector();
FVector recoilDirection = -forwardVector;
float recoilMagnitude = CalculateRecoilMagnitude();
GetCharacterMovement()->AddImpulse(recoilDirection * recoilMagnitude);
}
void APawn::Fire()
{
OnPawnFire();
}
#include "GameFramework/Controller.h"
#include "GameFramework/Pawn.h"
#include "GameFramework/Character.h"
UPROPERTY(EditDefaultsOnly, Category = "Weapon")
float RecoilSmoothingSpeed = 10.0f;
void AYourWeaponClass::ApplyRecoil(FVector RecoilDirection)
{
    if (Controller != nullptr && Controller->IsLocalPlayerController())
    {
FRotator ControllerRotation = Controller->GetControlRotation();
FVector RecoilDirectionInControlRotation = ControllerRotation.RotateVector(RecoilDirection);
FRotator TargetRotation = ControllerRotation + RecoilDirectionInControlRotation;
FRotator SmoothedRotation = FMath::RInterpTo(ControllerRotation, TargetRotation, DeltaTime, RecoilSmoothingSpeed);
Controller->SetControlRotation(SmoothedRotation);
    }
}    
void AMyCharacter::ApplyRecoil()
{
    FVector Direction = GetActorForwardVector();
    Direction.Normalize();

    // Calculate the vertical recoil compensation
    float VerticalRecoilCompensation = 1.0f;
    float VerticalRecoil = GetVerticalRecoil();
    if (bUseVerticalRecoilCompensation)
    {
        VerticalRecoilCompensation = fVerticalRecoilCompensation;
    }

    // Apply the recoil force
    FVector RecoilForce = Direction * (RecoilStrength * VerticalRecoilCompensation);
    Mesh->AddForce(RecoilForce);
}
UPROPERTY(EditDefaultsOnly, Category=Recoil)
float HorizontalRecoilMin;
UPROPERTY(EditDefaultsOnly, Category=Recoil)
float HorizontalRecoilMax;
UPROPERTY(EditDefaultsOnly, Category=Recoil)
float HorizontalRecoilSpeed;
float CurrentHorizontalRecoil;

void AYourWeaponClassName::ApplyHorizontalRecoil()
{
    float RecoilAmount = FMath::RandRange(HorizontalRecoilMin, HorizontalRecoilMax);
    FVector RecoilDirection = GetActorRightVector();
    
    // Apply recoil force to the weapon
    // ...
}

void AYourWeaponClassName::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
    
    // Update horizontal recoil based on recoil speed
    if (CurrentHorizontalRecoil > 0)
    {
        CurrentHorizontalRecoil -= HorizontalRecoilSpeed * DeltaTime;
        // Apply horizontal recoil offset to the weapon or character aim
        // ...
    }
}

void AYourWeaponClassName::Fire()
{
    // Trigger the weapon to fire
    
    // Apply horizontal recoil
    CurrentHorizontalRecoil = FMath::RandRange(HorizontalRecoilMin, HorizontalRecoilMax);
}

void AYourWeaponClassName::BeginPlay()
{
    Super::BeginPlay();
    
    // Other initialization
    
    CurrentHorizontalRecoil = 0.0f;
}
void AYourCharacter::ApplyVerticalRecoil()
{
    float RecoilAmount = FMath::FRandRange(VerticalRecoilMin, VerticalRecoilMax);
    FVector VerticalImpulse = GetActorUpVector() * RecoilAmount;
    GetCharacterMovement()->AddImpulse(VerticalImpulse, true);
}

void AYourCharacter::StartVerticalRecoil()
{
    GetWorld()->GetTimerManager().SetTimer(VerticalRecoilTimerHandle, this, &AYourCharacter::ApplyVerticalRecoil, VerticalRecoilTime, true);
}

void AYourCharacter::StopVerticalRecoil()
{
    GetWorld()->GetTimerManager().ClearTimer(VerticalRecoilTimerHandle);
}
void AWeapon::InitializeRecoilCompensation()
{
    CurrentHorizontalRecoilCompensation = 0.0f;
}
void AWeapon::ApplyHorizontalRecoilCompensation(float DeltaTime)
{
    CurrentHorizontalRecoilCompensation += HorizontalRecoilCompensationSpeed * DeltaTime;
    CurrentHorizontalRecoilCompensation = FMath::Clamp(CurrentHorizontalRecoilCompensation, 0.0f, MaxHorizontalRecoilCompensation);
}
void AWeapon::DecreaseHorizontalRecoilCompensation(float DeltaTime)
{
    CurrentHorizontalRecoilCompensation -= HorizontalRecoilCompensationDecreaseSpeed * DeltaTime;
    CurrentHorizontalRecoilCompensation = FMath::Clamp(CurrentHorizontalRecoilCompensation, 0.0f, MaxHorizontalRecoilCompensation);
}
float AWeapon::GetHorizontalRecoilCompensation() const
{
    return CurrentHorizontalRecoilCompensation;
}
AMyCharacter::AMyCharacter()
{
RecoilRange = FVector2D(500.f, 1000.f);
}

void AMyCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
    Super::SetupPlayerInputComponent(PlayerInputComponent);
PlayerInputComponent->BindAction("Fire", IE_Pressed, this, &AMyCharacter::ApplyRecoil);
}
void AMyCharacter::ApplyRecoil()
{
    if (GetCharacterMovement()->IsMovingOnGround())
    {
        FVector2D RecoilDirection = FVector2D(FMath::FRandRange(-1.f, 1.f), FMath::FRandRange(-1.f, 1.f)).GetSafeNormal();
        float RecoilForce = FMath::FRandRange(RecoilRange.X, RecoilRange.Y);

        FVector Impulse = FVector(RecoilDirection * RecoilForce, 0.f, 0.f);
        GetCharacterMovement()->AddImpulse(Impulse, true);
    }
}    
UPROPERTY(EditAnywhere, Category = "Weapon")
float RecoilForce = 1000.0f;
UPROPERTY(EditAnywhere, Category = "Weapon")
float RecoilTime = 1.0f;
float CurrentRecoilTime = 0.0f;
FVector RecoilDirection = FVector::ZeroVector;
void AYourCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
PlayerInputComponent->BindAction("Fire", IE_Pressed, this, &AYourCharacter::StartFire);
}
void AYourCharacter::StartFire()
{
if (CurrentRecoilTime <= 0.0f)
    {
RecoilDirection = GetRandomRecoilDirection();
GetCharacterMovement()->AddImpulse(RecoilDirection * RecoilForce, true);
CurrentRecoilTime = RecoilTime;
    }
}
FVector AYourCharacter::GetRandomRecoilDirection()
{
    FVector ViewVector = GetFollowCamera()->GetForwardVector();
return ViewVector;
}
void AYourCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
CurrentRecoilTime -= DeltaTime;
}            
UFUNCTION(BlueprintCallable)
void FireGun()
{
    PlayFireAnimation();
    SpawnBullet();
    FVector RecoilOffset = CalculateRecoilOffset();
    AddActorWorldOffset(RecoilOffset);
    AddActorWorldRotation(RecoilOffset.Rotation());
}
FVector AGun::CalculateRecoilOffset()
{
    float MaxRecoilOffsetX = GetDefault<AGun>()->MaxRecoilOffsetX;
    float MinRecoilOffsetX = GetDefault<AGun>()->MinRecoilOffsetX;
    float MaxRecoilOffsetY = GetDefault<AGun>()->MaxRecoilOffsetY;
    float MinRecoilOffsetY = GetDefault<AGun>()->MinRecoilOffsetY;
    float MaxRecoilRotationYaw = GetDefault<AGun>()->MaxRecoilRotationYaw;
    float MinRecoilRotationYaw = GetDefault<AGun>()->MinRecoilRotationYaw;
    float MaxRecoilRotationPitch = GetDefault<AGun>()->MaxRecoilRotationPitch;
    float MinRecoilRotationPitch = GetDefault<AGun>()->MinRecoilRotationPitch;
    float RandomOffsetX = FMath::RandRange(MinRecoilOffsetX, MaxRecoilOffsetX);
    float RandomOffsetY = FMath::RandRange(MinRecoilOffsetY, MaxRecoilOffsetY);
    float RandomRotationYaw = FMath::RandRange(MinRecoilRotationYaw, MaxRecoilRotationYaw);
    float RandomRotationPitch = FMath::RandRange(MinRecoilRotationPitch, MaxRecoilRotationPitch);
    FVector RecoilOffset(RandomOffsetX, RandomOffsetY, 0.0f);
    FRotator RecoilRotation(RandomRotationPitch, RandomRotationYaw, 0.0f);
    RecoilOffset = RecoilOffset.RotateAngleAxis(RecoilRotation.Pitch, FVector(1, 0, 0))
        .RotateAngleAxis(RecoilRotation.Yaw, FVector(0, 1, 0));

    return RecoilOffset;
}
float RecoilForceMagnitude;
FVector RecoilDirection;
float DesiredMovementAfterShotScalar;
void AMyCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
    if (RecoilForceMagnitude > 0.0f)
    {
        FVector RecoilForce = RecoilForceMagnitude * RecoilDirection;
        GetCharacterMovement()->AddImpulse(RecoilForce);
        RecoilForceMagnitude = 0.0f;
    }
}
void AMyCharacter::Shoot()
{
    RecoilForceMagnitude = CalculateRecoilForce();
    RecoilDirection = CalculateRecoilDirection();
    GetCharacterMovement()->MaxWalkSpeed = DesiredMovementAfterShotScalar * GetCharacterMovement()->GetMaxSpeed();
    APlayerCameraManager* CameraManager = GetWorld()->GetFirstPlayerController()->PlayerCameraManager;
CameraManager->CameraLensEffectScale = 1.0f;
CameraManager->CameraLensEffectFalloff = FVector2D(0.5f, 1.0f);
CameraManager->CameraLensEffectBase = FVector2D(0.0f, 0.0f);
CameraManager->bEnableOscillation = true;
CameraManager->OscillationBaseAmplitude = FVector2D(0.0f, 0.0f);
CameraManager->OscillationBaseFrequency = FVector2D(0.0f, 0.0f);
UPlayerInput* PlayerInput;
float RecoilXAxisInput = PlayerInputComponent->GetAxisValue("RecoilX");
bool bHasAimDownSightsRecoilReduction = false;
float AimDownSightsRecoilReductionAmount = 0.0f;

void ApplyRecoil()
{
    float RecoilAmount = 1.0f;

    if (bHasAimDownSightsRecoilReduction)
    {
        RecoilAmount *= (1.0f - AimDownSightsRecoilReductionAmount);
    }
    float RecoilCoefficient = GetDefault<UGameUserSettings>()->GetFloat("recoilCoefficient");
    UPROPERTY(EditAnywhere, Category = "Weapon")
float VerticalRecoilAmount = 1.0f;
void YourGunClass::HandleShoot()
{
FVector2D RecoilVector = FVector2D(FMath::RandRange(-VerticalRecoilAmount, VerticalRecoilAmount), 0.0f);
void ApplyRecoil(FVector RecoilVector)
   {
       FRotator CurrentRotation = GetActorRotation();
       FRotator RecoilRotation = FRotator(0, RecoilVector.Y, 0);
       FRotator NewRotation = CurrentRotation + RecoilRotation;
       SetActorRotation(NewRotation);
   }
   #include "GunVerticalNoRecoil.h"
AGunVerticalNoRecoil::AGunVerticalNoRecoil()
{
    PrimaryActorTick.bCanEverTick = true;
    RecoilAmount = 0.0f;
}
void AGunVerticalNoRecoil::BeginPlay()
{
    Super::BeginPlay();

    StartRotation = GetActorRotation();
}
void AGunVerticalNoRecoil::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
    FRotator NewRotation = StartRotation;
    NewRotation.Pitch += RecoilAmount * FMath::Clamp(DeltaTime * 20.0f, 0.0f, 1.0f);
    SetActorRotation(NewRotation);
    RecoilAmount = FMath::FInterpTo(RecoilAmount, 0.0f, DeltaTime, 5.0f);
}
void AGunVerticalNoRecoil::RecoilReset()
{
    RecoilAmount = 0.0f;
}
float AMyWeapon::CalculateVerticalRecoil()
{
    float VerticalRecoil = FMath::RandRange(0.0f, VerticalRecoilMax);
    return VerticalRecoil;
}