AActor* AYourCharacter::AutoTargetingFunction()
{
    TArray<AActor*> FoundActors;
    UGameplayStatics::GetAllActorsOfClass(GetWorld(), AYourTargetClass::StaticClass(), FoundActors);

    AActor* BestTarget = nullptr;
    float BestDistance = AutoTargetingRange;

    for (AActor* Target : FoundActors)
    {
        float Distance = FVector::Dist(Target->GetActorLocation(), GetActorLocation());
        if (Distance < BestDistance)
        {
            BestTarget = Target;
            BestDistance = Distance;
        }
    }

    return BestTarget;
}

class ACharacter : public AActor
{
    GENERATED_BODY()

public:
    // Apply damage to the character
    UFUNCTION(BlueprintCallable)
    void TakeDamage(float Damage);

private:
    float Health;
};

// Character.cpp
void ACharacter::TakeDamage(float Damage)
{
    // Apply damage to the character
    Health -= Damage;

    if (Health <= 0)
    {
        // Character dies
        Die();
    }
}

void ACharacter::Die()
{
    // Character death logic
}

void AyouCharacter::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);

    // Process bullet collision logic here based on current weapon
    // ...
}

void AyouCharacter::TakeDamage(float DamageAmount, FDamageEvent const& DamageEvent,
    AController* EventInstigator, AActor* DamageCauser)
{
    Super::TakeDamage(DamageAmount, DamageEvent, EventInstigator, DamageCauser);

    // Process death here based on health level
    // ...
}

AYourCharacter::AYourCharacter()
{
UPhysicsSettings* PhysicsSettings = GetDefault<UPhysicsSettings>();
PhysicsSettings->AirDensity = 0.0f;
}

void AYourPlayerControllerClass::SetupInputComponent()
{
    Super::SetupInputComponent();
InputComponent->BindAction("Reload", IE_Pressed, this, &AYourPlayerControllerClass::ReloadWeapon);
}

void AYourPlayerControllerClass::ReloadWeapon()
{
    AYourCharacterClass* YourCharacter = Cast<AYourCharacterClass>(GetPawn());
    if (YourCharacter)
    {
        if (bEnableQuickReload)
YourCharacter->SetAmmoCount(YourCharacter->GetMaxAmmoCount());
        }
        else
        {
YourCharacter->SetAmmoCount(YourCharacter->GetMaxAmmoCount());
        }
    }
}                    
void AYourCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
    Super::SetupPlayerInputComponent(PlayerInputComponent);
    PlayerInputComponent->AxisSettings["TurnRate"].Exponent = 1.0f;
    APlayerController* PC = GetWorld()->GetFirstPlayerController();
    PC->PlayerCameraManager->SetMaxRecoilCompensation(500.0f);
    APlayerController* PlayerController = GetWorld()->GetFirstPlayerController();
if(PlayerController)
{
  PlayerController->SetInputMouseSensitivity(X, Y);
}
APlayerController* PlayerController = GetWorld()->GetFirstPlayerController();
if(PlayerController)
{
  PlayerController->InputUpdateTime = X;
}
#include "YourCharacter.h"

// Sets default values
AYourCharacter::AYourCharacter()
{
    // Set default max turn rate
    MaxTurnRate = 45.0f;
}

// Called to bind functionality to input
void AYourCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
    Super::SetupPlayerInputComponent(PlayerInputComponent);

    PlayerInputComponent->BindAxis("Turn", this, &AYourCharacter::RotateCharacter);
}

// Function to rotate the character
void AYourCharacter::RotateCharacter(float Value)
{
    AddControllerYawInput(Value * MaxTurnRate * GetWorld()->GetDeltaSeconds());
}
void AYourCharacter::HandleHitDetection(FHitResult HitResult)
{
if (HitResult.GetComponent()->GetCollisionProfileName().ToString() == "MagicDamage")
    {
    if (HitResult.BoneName == "SpecificBoneName")
        {
        ApplyMagicDamage(100);
        }
    }
}
void AYourCharacter::ApplyMagicDamage(float DamageAmount)
{
Health -= DamageAmount;
#pragma once

#include "GameFramework/Character.h"
#include "YourPlayerCharacter.generated.h"

UCLASS()
class YOURPROJECT_API AYourPlayerCharacter : public ACharacter
{
    GENERATED_BODY()

public:
    AYourPlayerCharacter();
    UFUNCTION(BlueprintCallable)
    void SaveShotCountToINI();
    UPROPERTY(EditAnywhere, Category = "Player")
    int32 ShotCount;
};
#include "YourPlayerCharacter.h"
#include "HAL/FileManagerGeneric.h"

AYourPlayerCharacter::AYourPlayerCharacter()
{
    ShotCount = 0;
}

void AYourPlayerCharacter::SaveShotCountToINI()
{
    FConfigCacheIni Config(*GGameIni);
    Config.SetFloat(TEXT("PlayerSettings"), TEXT("ShotCount"), ShotCount, GGameIni);
}
#include "YourCharacter.h"

AYourCharacter::AYourCharacter()
{
AutoPossessPlayer = EAutoReceiveInput::Player0;
}

void AYourCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
    Super::SetupPlayerInputComponent(PlayerInputComponent);
    PlayerInputComponent->BindAction("AimAssist", IE_Pressed, this, &AYourCharacter::AimAssist);
    PlayerInputComponent->BindAxis("MoveRight", this, &AYourCharacter::MoveRight);
    PlayerInputComponent->BindAxis("MoveUp", this, &AYourCharacter::MoveUp);
}

void AYourCharacter::MoveRight(float Value)
{
AddMovementInput(GetActorRightVector(), Value);
}

void AYourCharacter::MoveUp(float Value)
{
AddMovementInput(GetActorForwardVector(), Value);
}

void AYourCharacter::AimAssist()
void AYourActor::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);

    if (TargetActor)
    {
    TargetActor = nullptr;
}

void AYourActor::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);

    if (TargetActor)
    {
    FVector CurrentLocation = GetActorLocation();
        FVector TargetLocation = TargetActor->GetActorLocation();
        FRotator TargetRotation = FRotationMatrix::MakeFromX(TargetLocation - CurrentLocation).Rotator();
        FRotator NewRotation = FMath::RInterpTo(GetActorRotation(), TargetRotation, DeltaTime, 5.0f);
        SetActorRotation(NewRotation);
    }
}