#include "CoreMinimal.h"```cpp
#include "GameFramework/Actor.h"
#include "MyCharacter.generated.h"

UCLASS()
class MYPROJECT_API AMyCharacter : public AActor
{
    GENERATED_BODY()

public:
    AMyCharacter();

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Weapon")
    TSubclassOf<class AWeapon> WeaponClass;

    UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Weapon")
    class AWeapon* CurrentWeapon;

protected:
    virtual void BeginPlay() override;

public:
    virtual void Tick(float DeltaTime) override;

    UFUNCTION(BlueprintCallable)
    void PerformMeleeAttack();

private:
    UFUNCTION()
    void OnMeleeAttackStart();

    UFUNCTION()
    void OnMeleeAttackEnd();

    UFUNCTION(Server, Reliable)
    void ServerPerformMeleeAttack();

    UFUNCTION(NetMulticast, Reliable)
    void MulticastOnMeleeAttackStart();

    UFUNCTION(NetMulticast, Reliable)
    void MulticastOnMeleeAttackEnd();
};
#include "GameFramework/Actor.h"
#include "MyDamageComponent.generated.h"

UCLASS(ClassGroup = (Custom), meta = (BlueprintSpawnableComponent))
class YOURGAME_API UMyDamageComponent : public UActorComponent
{
    GENERATED_BODY()

public:
    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Damage")
    float MaxDamage;

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Damage")
    float MaxDistance;

    UFUNCTION(BlueprintCallable, Category = "Damage")
    void ApplyDamage(AActor* Target);

protected:
    virtual void InitializeComponent() override;
};
UFUNCTION(BlueprintCallable, Category = "Combat")
float CalculateMaxBodyDamage(float BaseDamage, float HeadDamageMultiplier, float BodyDamageMultiplier, float LimbDamageMultiplier)
{
    float BodyDamage = BaseDamage * BodyDamageMultiplier;
    return BodyDamage;
}
UCLASS()
class YOURPROJECT_API AMyGunActor : public AActor
{
    GENERATED_BODY()

public:
    UPROPERTY(EditAnywhere, Category = "Weapon")
    float DamagePerBullet;

    UPROPERTY(EditAnywhere, Category = "Weapon")
    float FireRate;

    UPROPERTY(EditAnywhere, Category = "Weapon")
    float MaxDamageDistance;

    UPROPERTY(EditAnywhere, Category = "Weapon")
    float MaxDamageMultiplier;

    UFUNCTION(BlueprintCallable, Category = "Weapon")
    float CalculateDamageAtDistance(float Distance) const;

    UFUNCTION(BlueprintCallable, Category = "Weapon")
    float CalculateMaxDamage() const;
};
UPROPERTY(EditDefaultsOnly, Category = "DamageMax")
float DamageMultiplier;
#pragma once

#include "CoreMinimal.h>

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MyProjectile.generated.h"

UCLASS()
class YOURPROJECT_API AMyProjectile : public AActor
{
    GENERATED_BODY()

public:
    // Sets default values for this actor's properties
    AMyProjectile();

protected:
    // Called when the game starts or when spawned
    virtual void BeginPlay() override;

public:
    // Called every frame
    virtual void Tick(float DeltaTime) override;

    UFUNCTION()
    void OnHit(AActor* SelfActor, AActor* OtherActor, FVector NormalImpulse, const FHitResult& Hit);

    UPROPERTY(VisibleDefaultsOnly, Category = Projectile)
    class USphereComponent* CollisionComponent;

    UPROPERTY(VisibleAnywhere, Category = Movement)
    class UProjectileMovementComponent* ProjectileMovementComponent;

    UPROPERTY(EditAnywhere, Category = Damage)
    float MaxDamage;

    UPROPERTY(EditAnywhere, Category = Damage)
    float MinDamage;

    UPROPERTY(EditAnywhere, Category = Damage)
    float MaxRange;

private:
    UFUNCTION()
    void DealDamage(FHitResult& HitResult);
};
UFUNCTION(BlueprintCallable, Category = "Gameplay")
void CalculateMaxAimDamage();
#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "CircleDamageRate.generated.h"

UCLASS()
class YOURPROJECTNAME_API ACircleDamageRate : public AActor
{
    GENERATED_BODY()

public: 
    // Sets default values for this actor's properties
    ACircleDamageRate();

protected:
    // Called when the game starts or when spawned
    virtual void BeginPlay() override;

public: 
    // Called every frame
    virtual void Tick(float DeltaTime) override;

    UPROPERTY(EditAnywhere, Category = "Circle Damage")
    float Radius;

    UPROPERTY(EditAnywhere, Category = "Circle Damage")
    float DamageRate;

    UFUNCTION(BlueprintCallable, Category = "Circle Damage")
    void ApplyDamageInCircle(const FVector& Center);

};
#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MyDamageDealer.generated.h"

UCLASS()
class YOURPROJECT_API AMyDamageDealer : public AActor
{
    GENERATED_BODY()

public:    
    AMyDamageDealer();

    UFUNCTION(BlueprintCallable)
    void DealDamageInArea(FVector Location, float Radius, float DamageAmount);

protected:
    UPROPERTY(EditDefaultsOnly, Category = "Damage")
    TSubclassOf<UDamageType> DamageType;

    UPROPERTY(EditDefaultsOnly, Category = "Damage")
    AController* InstigatorController;
};